/* Spoiala Marian 333CB - Proiect PM - Tank WI-FI */

#include "lcd.h"
#include "usart.h"

#define F_CPU 16000000
#define CHUNK_SIZE 32
#include <stdlib.h>
#include <string.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <ctype.h>

char pagina_html[] = "<html><body><style>.mine{width: 100px;}</style><title>Tanc Wi-FI</title>\
<input type=\"range\" id=\"my\" style=\"-webkit-appearance: slider-vertical\" value=\"0\" min=\"0\" max=\"255\"/>\
<table> <tr> <td> </td><td> <button class=\"mine\" onclick=\"comanda('U')\">Inainte</button> </td><td> </td></tr><tr> <td>\
<button class=\"mine\" onclick=\"comanda('L')\">Stanga</button> </td><td> <button class=\"mine\" onclick=\"comanda('S')\">Stop</button>\
</td><td> <button class=\"mine\" onclick=\"comanda('R')\">Dreapta</button> </td></tr><tr> <td> </td><td> <button class=\"mine\" onclick=\"comanda('D')\">\
Inapoi</button> </td><td> </td></tr></table><script>var http=new XMLHttpRequest();\
function comanda(data){http.abort();\
http.open(\"POST\", \"http://192.168.4.1:8001?comanda=\" + data + document.getElementById(\"my\").value, true); http.send();};</script></body></html>";

int recv_alert = 0;
int timeout = 0;
int gl_size = 398;
char gl_answer[400];
int gl_end = 0;
char cx;

//Timer1 - Intrerupere pentru timeout, astfel incat daca dureaza 
//prea mult sa primesc raspuns la o comanda, reinitializez totul
ISR(TIMER1_COMPA_vect)
{
    timeout = 1;
}

void toggle_led()
{
	PORTB = PORTB ^ (1 << PB0);	// inversam starea ledului
}

//Rutina de tratare a intreruperii pentru receive pe seriala
//Necesita activarea bitului RXCIEn
ISR(USART0_RX_vect) {
	
	cx = UDR0;		
	
	//Setez alerta de receive
	recv_alert = 1;
	
	//Daca se umple bufferul de raspuns, fac flush. Vor fi ignorate.
	//Acesta se va reseta cand se citeste primul request din el.
	if (gl_end < gl_size) {
		gl_answer[gl_end++] = cx;
		gl_answer[gl_end] = '\0';
	}
	
	//toggle_led();
}

/* Initializeaza timer 1 */
void timer1_init()
{
    TCCR1B |= (1 << CS12)| (1 << CS10) | (1 << WGM12);

	//5 secunde - clock = 16MHz
	OCR1A = 3125;
	//1 secunda - clock = 16MHz
    //OCR1A = 15625;
}

void timer1_activate() {
	TIMSK1 |= ( 1<< OCIE1A);
}

void timer1_deactivate() {
	TIMSK1 &= ~( 1<< OCIE1A);
}

int initialize_wifi() {
	
	//Resetez wifi - este necesar un delay pt ca dureaza pana se reseteaza
	gl_end = 0;
	USART0_print("AT+RST\r\n");
	_delay_ms(2000);
	gl_answer[gl_end] = '\0';
	if (!strstr(gl_answer, "ready")) {
		return 0;
	}
	
	//Schimb modul de rulare ca AP
	gl_end = 0;
	USART0_print("AT+CWMODE=3\r\n");
	_delay_ms(1000);
	gl_answer[gl_end] = '\0';
	if (!strstr(gl_answer, "OK") && !strstr(gl_answer,"no change")) {
		return 0;
	}
		
	//Configurez ca multi conexiune
	gl_end = 0;
	USART0_print("AT+CIPMUX=1\r\n");
	_delay_ms(300);
	gl_answer[gl_end] = '\0';
	if (!strstr(gl_answer, "OK")) {
		return 0;
	}
	
	//Setez ca server care asculta pe port 80
	gl_end = 0;
	USART0_print("AT+CIPSERVER=1,8001\r\n");
	_delay_ms(500);
	gl_answer[gl_end] = '\0';
	if (!strstr(gl_answer, "OK")) {
		return 0;
	}

	//Aflare adresa IP
	//USART0_print("AT+CIFSR\r\n");
	
	return 1;
}

void process_requests() {
	int i;
	char sesiune;
	char comanda[50];
	char nr[5];
	char portiune_html[CHUNK_SIZE+4];
	char *ptr;
	char *ptr_com;
	int send_size;
	int old_0A, old_0B, old_2A, old_2B;
	
	//Daca a primit ceva pe seriala
	if (recv_alert == 1) {
		_delay_ms(100);
		
		ptr = strstr(gl_answer, "+IPD");
		//Daca sunt date
		if (strstr(gl_answer, "+IPD")) {
			
			//Ii acord timp sa primeasca un request HTTP
			_delay_ms(200);
			
			sesiune = *(ptr+5);
			
			//Semnalez cu un led ca este un request
			toggle_led();
			
			//Daca este HTTP GET pentru executare comanda, caut parametrul
			if (strstr(gl_answer, "comanda=")) {
				ptr_com = strstr(gl_answer, "comanda=");
				
				//Aflare viteza
				int k, p;
				char vit_chr[4];
				k=0;
				for (p=9; p<=11; p++) {
					if(isdigit(*(ptr_com+p))) {
						k++;
					}
				}
				
				long int vit;
				strncpy(vit_chr, ptr_com+9, k);
				vit_chr[k+1] = '\0';
				vit = atoi(vit_chr);
				//
				
				if (*(ptr_com+8) == 'U') {					
					//Motor stanga
					OCR0A = vit; //inainte
					OCR0B = 0; //inapoi
					
					//Motor dreapta
					OCR2A = vit; //inainte
					OCR2B = 0; //inapoi
					_delay_ms(300);
				}
				else if (*(ptr_com+8) == 'D') {
					//Motor stanga
					OCR0A = 0; //inainte
					OCR0B = vit; //inapoi
					
					//Motor dreapta
					OCR2A = 0; //inainte
					OCR2B = vit; //inapoi	
					_delay_ms(300);							
				}
				else if (*(ptr_com+8) == 'L') {					
					old_0A = OCR0A; old_0B = OCR0B; old_2A = OCR2A; old_2B = OCR2B;
					
					//Motor stanga
					OCR0A = vit; //inainte
					OCR0B = 0; //inapoi
					
					//Motor dreapta
					OCR2A = 0; //inainte
					OCR2B = vit; //inapoi
					
					_delay_ms(1000);
					
					//Motor stanga
					OCR0A = old_0A; //inainte
					OCR0B = old_0B; //inapoi
					
					//Motor dreapta
					OCR2A = old_2A; //inainte
					OCR2B = old_2B; //inapoi
				
				}
				else if (*(ptr_com+8) == 'R') {
					old_0A = OCR0A; old_0B = OCR0B; old_2A = OCR2A; old_2B = OCR2B;
					
					//Motor stanga
					OCR0A = 0; //inainte
					OCR0B = vit; //inapoi
					
					OCR2A = vit;//inainte
					OCR2B = 0;//inapoi
					_delay_ms(1000);
					
					//Motor stanga
					OCR0A = old_0A; //inainte
					OCR0B = old_0B; //inapoi
					
					//Motor dreapta
					OCR2A = old_2A; //inainte
					OCR2B = old_2B; //inapoi				
				}
				else if (*(ptr_com+8) == 'S') {
					//Motor stanga
					OCR0A = 0; //inainte
					OCR0B = 0; //inapoi
					
					//Motor dreapta
					OCR2A = 0; //inainte
					OCR2B = 0; //inapoi
					_delay_ms(300);
				}
			}
			//Daca este HTTP GET pentru obtinere pagina
			//Fiecare HTTP request pt pagina, face cerere pt favicon care 
			//nu ma intereseaza
			else if (!strstr(gl_answer, "favicon")){
				_delay_ms(2000);
				
				int html_page_size = strlen(pagina_html);
				itoa(html_page_size, nr, 10);
				
				//Trimit pe seriala numarul de caractere care urmeaza transmis
				strncpy(comanda, "AT+CIPSEND=", 11);
				comanda[11] = sesiune; comanda[12]=',';
				if (html_page_size>1000) {
					strncpy(comanda+13, nr, 4);
					comanda[17]='\r'; comanda[18]='\n';
					comanda[19]='\0';
				}
				if (html_page_size>100) {
					strncpy(comanda+13, nr, 3);
					comanda[16]='\r'; comanda[17]='\n';
					comanda[18]='\0';
				}
				else if (html_page_size>10) {
					strncpy(comanda+13, nr, 2);
					comanda[15]='\r'; comanda[16]='\n';
					comanda[17]='\0';
				}
				else {
					strncpy(comanda+13, nr, 1);
					comanda[14]='\r'; comanda[15]='\n';
					comanda[16]='\0';
				}
					
				//Daca nu pun delay-uri nu apuca sa se faca transmisia wifi.
				USART0_print(comanda);
				_delay_ms(200);
					
				//Execut comenzi care trimit bucati din pagina HTML de dim CHUNK_SIZE
				for (i=0; i<html_page_size; i+=CHUNK_SIZE) {
					if (i+CHUNK_SIZE >= html_page_size) {
						send_size = html_page_size%CHUNK_SIZE;						
					}
					else {
						send_size = CHUNK_SIZE;
					}					
					
					//Ultima bucata nu e CHUNK_SIZE
					strncpy(portiune_html, pagina_html+i, send_size);
					portiune_html[send_size]='\0';
					USART0_print(portiune_html);
					_delay_ms(200);
				}		
				
			}			
			
			//Execut comanda de inchidere TCP
			strncpy(comanda, "AT+CIPCLOSE=", 12);
			comanda[12] = sesiune; comanda[13]='\r'; comanda[14]='\n';
			comanda[15]='\0';
			USART0_print(comanda);
			_delay_ms(500);
			
			gl_end = 0;
			recv_alert = 0;
			gl_answer[gl_end] = '\0';
		}		
	}
}

//Initializare timer0 ca FAST PWM
void timer0_init()
{
    TCCR0A |= (1 << WGM00) | (1<<WGM01) | (1 << COM0A1) | (1<<COM0B1);
    TCCR0B |= (1 << CS00);
    OCR0A = 0;
	OCR0B = 0;
}

//Initializare timer2 ca FAST PWM
void timer2_init()
{
    TCCR2A |= (1 << WGM20) | (1<<WGM21) | (1 << COM2A1) | (1<<COM2B1);
    TCCR2B |= (1 << CS20);
    OCR2A = 0;
	OCR2B = 0;
}

int main()
{
	int ok;
	// Activez intreruperile global
	sei();
	
	//Setez pin 0 port B ca pin de iesire pt testare
	DDRB |= (1 << PB0);
	
	//Setez pinii pentru FastPWM
	DDRB |= (1 << PB3) | (1 << PB4);
	DDRD |= (1 << PD6) | (1<<PD7);
	
	timer0_init();
	timer2_init();
	
	//Comunicare seriala 9600 baud rate
	USART0_init(103);	
		
	gl_answer[399] = '\0';
	ok = 0;
	while(1) {
		//Daca ok==0, se incearca initializare WI-FI
		while(ok == 0) {		
			if (initialize_wifi() == 1) {
				ok = 1;
				toggle_led();
			}
		}
		//Daca ok==1, intra pe modul de ascultare requesturi HTTP
		while(ok == 1) {
			process_requests();
		}
	}
	
    return 0;
}